package com.malk.aiwei.service.impl;

import com.alibaba.fastjson.JSON;
import com.malk.aiwei.service.AWClint;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.dingtalk.DDR_New;
import com.malk.service.aliwork.YDClient;
import com.malk.service.aliwork.YDService;
import com.malk.utils.UtilMap;
import com.malk.utils.UtilString;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
@Slf4j
public class AWImplClient implements AWClint {


    @Autowired
    private YDClient ydClient;

    @Autowired
    private YDService ydService;

    /**
     * 交付物审批 [ppExt: 宜搭附件传递 downloadUrl 和 name 即可实现在线预览]
     */
    @Override
    public String approve(String tCode, String pCode, List<Map> docs, String taskId, String operatorId, String userId) {

        log.info("交付物审批, {}", JSON.toJSONString(UtilMap.map("tCode, pCode, docs, taskId, operatorId, userId", tCode, pCode, docs, taskId, operatorId, userId)));
        // 工作流tb配置了流转逻辑, 冗余稳定性
        DDR_New ddr_new = ydClient.queryData(YDParam.builder()
                .formUuid("FORM-812FD46AF391449A8F206EDB3221B38840UQ")
                .searchFieldJson(JSON.toJSONString(UtilMap.map("selectField_lqxuswzd, selectField_lqxuswze", tCode, pCode)))
                .instanceStatus("RUNNING")
                .build(), YDConf.FORM_QUERY.retrieve_search_process);
        if (ddr_new.getTotalCount() > 0) {
            return "";
        }
        // 查询任务表与项目表
        List<Map> tList = ydService.queryDataList_FormData("FORM-A25299893F614A6EAA672514D3A76BB0QDBF", UtilMap.map("textField_lqxu8st8", tCode));
        if (tList.size() == 0) {
            return "未配置交付物审批.";
        }
        List<Map> pList = ydService.queryDataList_FormData("FORM-141E21DF183846028E21727CE43CD1C75CLZ", UtilMap.map("textField_lqxtykce", pCode));
        if (pList.size() == 0) {
            return "未匹配到项目主数据";
        }
        List<Map> roles = (List<Map>) pList.get(0).get("tableField_lqxtykcf");
        Map formData = UtilMap.map("selectField_lqxuswzd, selectField_lqxuswze, textField_lqxuc9m4", tCode, pCode, UtilMap.getString(pList.get(0), "textField_lqxtykcd"));
        formData.put("attachmentField_lqxtebtq", docs);
        // 匹配任务编码与项目角色
        List<Map<String, String>> compIds = Arrays.asList(UtilMap.map("tsRole, prRole, prEmp", "selectField_lqxu76th, selectField_lqxu76th, employeeField_lqxtebtw"),
                UtilMap.map("tsRole, prRole, prEmp", "selectField_lqxu76tj, selectField_lqxuw78q, employeeField_lqxtebtx"), // 任务表角色, 交付物评审表: 角色, 审批人
                UtilMap.map("tsRole, prRole, prEmp", "selectField_lqxu76tk, selectField_lqxuw78r, employeeField_lqxtebty"));
        for (Map compId : compIds) {

            Optional optional = roles.stream().filter(item -> UtilString.equal(tList.get(0).get(compId.get("tsRole")), item.get("selectField_lqxu6bgf"))).findAny();
            if (optional.isPresent()) {
                Map detail = (Map) optional.get();
                formData.put(compId.get("prRole"), detail.get("selectField_lqxu6bgf"));
                formData.put(compId.get("prEmp"), detail.get("employeeField_lqxtykch_id"));
            }
        }
        // 发起评审
        UtilMap.putAll(formData, UtilMap.map("textField_lr3dlwsa, textField_lr3er4qb", taskId, operatorId));
        String instanceId = (String) ydClient.operateData(YDParam.builder()
                .formUuid("FORM-812FD46AF391449A8F206EDB3221B38840UQ")
                .processCode("TPROC--RJC66SC1NEFHXJ0H770K0CF4WN1K21HQ706RL5")
                .formDataJson(JSON.toJSONString(formData))
                .userId(userId)
                .build(), YDConf.FORM_OPERATION.start);
//        return "记录: " + "https://kabom7.aliwork.com/APP_H7WUJTKB448F9IBDC6C4/processDetail?procInsId=" + instanceId;
        return "记录: " + "https://yida.awinic.com/APP_R5EBUF2FPN3Y8DRF93M4/processDetail?procInsId=" + instanceId;
    }

    /**
     * 检查项check
     */
    @Override
    public String check(String tCode, String pCode, String taskId, String operatorId, String userId) {

        log.info("交付物审批, {}", JSON.toJSONString(UtilMap.map("tCode, pCode, taskId, operatorId, userId", tCode, pCode, taskId, operatorId, userId)));
        // 是否存在chek记录过滤
        List<Map> cList = ydService.queryDataList_FormData("FORM-7A50EA1991CE4494949FF7E1D4592FFBOVUS", UtilMap.map("selectField_lqxuswzd, selectField_lqxuswze", tCode, pCode));
        if (cList.size() > 0) {
            return "";
        }
        // 查询任务编码检查项与任务
        List<Map> tList = ydService.queryFormData_all(YDParam.builder()
                .formUuid("FORM-1A5D4D7FBF88409B956EBE51F9342A6BKOLP")
                .searchFieldJson(JSON.toJSONString(UtilMap.map("textField_lqxu8st8", tCode)))
                .build());
        if (tList.size() == 0) {
            return "未配置检查项.";
        }
        List<Map> pList = ydService.queryDataList_FormData("FORM-141E21DF183846028E21727CE43CD1C75CLZ", UtilMap.map("textField_lqxtykce", pCode));
        if (pList.size() == 0) {
            return "未匹配到项目主数据.";
        }
        Map formData = UtilMap.map("selectField_lqxuswzd, selectField_lqxuswze, textField_lqxuc9m4", tCode, pCode, UtilMap.getString(pList.get(0), "textField_lqxtykcd"));
        // 填充检查项目, 填充子表
        List<Map> checkList = tList.stream().map(item -> UtilMap.map("textField_lqxxgj4u", item.get("textField_lqxxgj4u"))).collect(Collectors.toList());
        formData.put("tableField_lqxxgj4s", checkList);
        // 发起评审
        UtilMap.putAll(formData, UtilMap.map("textField_lr3dlwsa, textField_lr3er4qb", taskId, operatorId));
        String instanceId = (String) ydClient.operateData(YDParam.builder()
                .formUuid("FORM-7A50EA1991CE4494949FF7E1D4592FFBOVUS")
                .processCode("TPROC--AKA66FB1XHFHN82QEFIQ591SNJ5W3AMBJ06RL9")
                .formDataJson(JSON.toJSONString(formData))
                .userId(userId)
                .build(), YDConf.FORM_OPERATION.start);
        return "记录: " + "https://yida.awinic.com/APP_R5EBUF2FPN3Y8DRF93M4/processDetail?procInsId=" + instanceId;
    }
}
